﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    internal class DrawningObject : IDrawningObject
    {
        public DrawningObject(DrawningAirplane airplane)
        {
            Airplane = airplane;
        }

        public float Step => Airplane?.Airplane?.Step ?? 0;

        public DrawningAirplane Airplane { get; private set; }

        public RectangleF GetCurrentPosition()
        {
            return Airplane.GetCurrentPosition();
        }

        public void MoveObject(Direction direction)
        {
            Airplane?.MoveTransport(direction);
        }

        public void SetObject(int x, int y, int width, int height)
        {
            Airplane?.SetPosition(x, y, width, height);
        }

        public void DrawObject(Graphics g)
        {
            Airplane?.DrawTransport(g);
        }

        public string GetInfo() => Airplane?.GetDataForSave() ?? string.Empty;

        public static IDrawningObject Create(string data) => new DrawningObject(data.CreateDrawningAirplane());

        public bool Equals(IDrawningObject? other)
        {
            if (other is not DrawningObject otherAirplane)
            {
                return false;
            }
            var entity = Airplane.Airplane;
            var otherEntity = otherAirplane.Airplane.Airplane;
            if (entity.GetType() != otherEntity.GetType() ||
                entity.Speed     != otherEntity.Speed     ||
                entity.Weight    != otherEntity.Weight    ||
                entity.BodyColor != otherEntity.BodyColor)
            {
                return false;
            }
            if (entity      is EntityAirBomber entityAirBomber && 
                otherEntity is EntityAirBomber otherEntityAirBomber && (
                entityAirBomber.HasBombs     != otherEntityAirBomber.HasBombs ||
                entityAirBomber.DopColor     != otherEntityAirBomber.DopColor ||
                entityAirBomber.HasFuelTanks != otherEntityAirBomber.HasFuelTanks))
            {
                return false;
            }
            return true;
        }
    }
}
