﻿namespace AirBomber
{
    /// <summary>
    /// Форма создания объекта
    /// </summary>
    public partial class FormAirplaneConfig : Form
    {
        /// <summary>
        /// Переменная-выбранная самолет
        /// </summary>
        DrawningAirplane _airplane = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawningAirplane> EventAddAirplane;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormAirplaneConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (s, e) => Close();
        }
        /// <summary>
        /// Отрисовать самолет
        /// </summary>
        private void DrawAirplane()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _airplane?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _airplane?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev"></param>
        public void AddEvent(Action<DrawningAirplane> ev)
        {
            if (EventAddAirplane == null)
            {
                EventAddAirplane = new(ev);
            }
            else
            {
                EventAddAirplane += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _airplane = new DrawningAirplane((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelModifiedObject":
                    _airplane = new DrawningAirBomber((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black,
                        checkBoxHasBombs.Checked, checkBoxHasFuelTanks.Checked);
                    break;
            }
            DrawAirplane();
        }
        /// <summary>
        /// Отправляем цвет с панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Установливает статус перемещаемых данных
        /// </summary>
        /// <param name="e"></param>
        /// <param name="needTypeData">Тип которого должны соответствовать перемещаемые данные</param>
        /// <param name="condition">условие на возможность копирования данных</param>
        private void setDragEffect(DragEventArgs e, Type needTypeData, bool condition)
        {
            if (e.Data.GetDataPresent(needTypeData) && condition)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelBaseColor_DragEnter(object sender, DragEventArgs e) => setDragEffect(e, typeof(Color), _airplane != null);
        private void  LabelDopColor_DragEnter(object sender, DragEventArgs e) => setDragEffect(e, typeof(Color), _airplane != null && _airplane is DrawningAirBomber);
        /// <summary>
        /// Принимаем основной цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = (Color)e.Data.GetData(typeof(Color));
            if (_airplane is DrawningAirBomber)
            {
                _airplane = ((DrawningAirBomber)_airplane).Copy(bodyColor: color);
            }
            else if (_airplane is DrawningAirplane)
            {
                _airplane = _airplane.Copy(bodyColor: color);
            }
            DrawAirplane();
        }
        /// <summary>
        /// Принимаем дополнительный цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = (Color)e.Data.GetData(typeof(Color));
            var airplane = _airplane as DrawningAirBomber;
            if (airplane != null)
            {
                _airplane = airplane.Copy(dopColor: color);
            }
            DrawAirplane();
        }
        /// <summary>
        /// Добавление самолета
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddAirplane?.Invoke(_airplane);
            Close();
        }
    }
}